;;; hs-show-html-mail.el --- View HTML-parts in a graphical browser.

;; Copyright (C) 2005 Hynek Schlawack

;; Author: Hynek Schlawack <hynek@ularx.de>
;; Version: $Id: hs-show-html-mail.el 12 2005-12-15 15:44:30Z hynek $
;; Keywords: mail news

;; This file is NOT part of GNU Emacs.

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; Scans the actual article in Gnus for HTML-parts and shows them using
;; browse-url.

;;; Code:

(defun hs-show-html-list (LIST)
  "View all HTML-parts from 'LIST'.
Recurse into multiparts."

  (let ((showed))
    ;; Find and show the html-parts.
    (dolist (elt LIST)
      ;; If HTML, show it
      (cond ((and (listp elt)
		  (bufferp (car elt))
		  (string-match "text/html" (car (mm-handle-type elt))))
	     (let ((tmp (format "%s-%s.html"
				(make-temp-name "/tmp/gnus-html-")
				(user-login-name))))
	       (mm-save-part-to-file elt tmp)
	       (browse-url tmp)
	       (setq showed t)))
	    ;; If multipart, recurse
	    ((and (listp elt)
		  (stringp (car elt))
		  (string-match "multipart/" (substring (car elt) 0 10))
		  (setq showed (or showed (hs-show-html-list elt)))))))
    showed))

(defun hs-show-html-mail ()
  "View HTML-parts of the current Gnus-Article in a graphical browser."
  (interactive)
  
  (save-window-excursion
    ;; Open raw article and select the buffer
    (gnus-summary-show-article t)
    (gnus-summary-select-article-buffer)
    (let ((parts (mm-dissect-buffer t t)))

      ;; If singlepart, enforce a list.
      (when (and (bufferp (car parts))
		 (stringp (car (mm-handle-type parts))))
	(setq parts (list parts)))

      ;; Process the list
      (unless (hs-show-html-list parts)
	(message "Mail doesn't contain a HTML-part!"))

      (gnus-summary-show-article))))
    
(provide 'hs-show-html-mail)

;;; hs-show-html-mail.el ends here